import org.junit.jupiter.api.Test;
import java.io.ByteArrayOutputStream;
import java.io.PrintStream;
import static org.junit.jupiter.api.Assertions.*;

class InspectorTest {
    @Test
    void testGetSuperclass(){
        Inspector inspector = new Inspector();
        testingObj testObj = new testingObj("testing");
        Class <?> _class = testObj.getClass();

        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        System.setOut(new PrintStream(outputStream));

        inspector.getSuperclass(_class, testObj, true, 1);

        System.setOut(System.out);
        String output = outputStream.toString();
        assertTrue(output.contains("SuperClass: java.lang.Object"));
        assertTrue(output.contains("SuperClass: NONE"));
    }

    @Test
    void testGetSuperclass2(){ // recursive = false should still contain super class
        Inspector inspector = new Inspector();
        testingObj testObj = new testingObj("testing");
        Class <?> _class = testObj.getClass();

        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        System.setOut(new PrintStream(outputStream));

        inspector.getSuperclass(_class, testObj, false, 1);

        System.setOut(System.out);
        String output = outputStream.toString();
        assertTrue(output.contains("SuperClass: java.lang.Object"));
        assertTrue(output.contains("SuperClass: NONE"));
    }

    @Test
    void testGetInterface(){
        Inspector inspector = new Inspector();
        testingObj2 testObj2 = new testingObj2();
        Class <?> _class = testObj2.getClass();

        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        System.setOut(new PrintStream(outputStream));

        inspector.getInterface(_class, testObj2, true, 1);

        System.setOut(System.out);
        String output = outputStream.toString();
        assertTrue(output.contains("INTERFACE"));
        assertTrue(output.contains("Name: testingInterface"));
    }

    @Test
    void testGetInterface2(){ //test interface with recursive = false
        Inspector inspector = new Inspector();
        testingObj2 testObj2 = new testingObj2();
        Class <?> _class = testObj2.getClass();

        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        System.setOut(new PrintStream(outputStream));

        inspector.getInterface(_class, testObj2, false, 1);

        System.setOut(System.out);
        String output = outputStream.toString();
        assertTrue(output.contains("INTERFACE"));
        assertTrue(output.contains("Name: testingInterface"));
    }

    @Test
    void testSimpleClass() {
        Inspector inspector = new Inspector();
        simpleClass simpleObj = new simpleClass();
        Class <?> _class = simpleObj.getClass();

        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        System.setOut(new PrintStream(outputStream));

        inspector.inspect(simpleObj, false);

        System.setOut(System.out);
        String output = outputStream.toString();

        assertTrue(output.contains("METHODS"));
        assertTrue(output.contains("Name: setNumber"));
        assertTrue(output.contains("FIELD"));
        assertTrue(output.contains("Name: number"));
    }

    @Test
    void testGetConstructor(){ //test interface with recursive = false
        Inspector inspector = new Inspector();
        testingObj3 testObj3 = new testingObj3(93);
        Class <?> _class = testObj3.getClass();

        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        System.setOut(new PrintStream(outputStream));

        inspector.getConstructor(_class, testObj3, false, 1);

        System.setOut(System.out);
        String output = outputStream.toString();
        assertTrue(output.contains("CONSTRUCTOR"));
        assertTrue(output.contains("Name: testingObj3"));
    }

    @Test
    void testNONE(){

        Inspector inspector = new Inspector();
        emptyClass emptyObj = new emptyClass();
        Class <?> _class = emptyObj.getClass();

        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        System.setOut(new PrintStream(outputStream));

        inspector.getConstructor(_class, emptyObj, false, 1);

        System.setOut(System.out);
        String output = outputStream.toString();
        assertTrue(output.contains("Exceptions-> NONE"));
        assertTrue(output.contains("Parameter Types-> NONE"));
        // this is for testing after the refactors if the decomposed conditionals didn't change anything
    }
}